10#ifndef IMPACTX_APERTURE_H
11#define IMPACTX_APERTURE_H
43 static constexpr auto type =
"Aperture";
96 if (aperture_x > 0_prt) {
99 throw std::runtime_error(
"Aperture: aperture_x must be > 0!");
102 if (aperture_y > 0_prt) {
105 throw std::runtime_error(
"Aperture: aperture_y must be > 0!");
109 throw std::runtime_error(
"Aperture: shift_odd_x can only be used if repeat_y is set, too!");
117 using BeamOptic::operator();
128 Alignment::compute_constants(refpart);
143 template<
typename T_Real=amrex::ParticleReal,
typename T_IdCpu=u
int64_t>
161 using VBool =
decltype(x>0_prt);
174 sx += shift_x ?
dx : 0_prt;
189 VBool inside_aperture;
193 case Shape::rectangular:
194 inside_aperture = (powi<2>(u) <= 1_prt && powi<2>(
v) <= 1_prt);
197 case Shape::elliptical:
198 inside_aperture = (powi<2>(u) + powi<2>(
v) <= 1_prt);
202 inside_aperture = VBool{
true};
208 auto const invalid_mask =
m_action == Action::transmit ? !inside_aperture : inside_aperture;
213 using Thin::operator();
231 template<
typename T_Real=amrex::ParticleReal,
typename T_IdCpu=u
int64_t>
250 (*this)(x, y,
t, px, py, pt, idcpu, refpart);
254 using LinearTransport::operator();
#define AMREX_FORCE_INLINE
#define AMREX_GPU_HOST_DEVICE
#define IMPACTX_PUSH_EXTERN_TEMPLATE(ElementType)
Definition PushAll.H:78
T_ParticleType ParticleType
amrex_particle_real ParticleReal
constexpr T powi(T x) noexcept
__host__ __device__ detail::where_expression< T > where(bool const mask, T &value)
__host__ __device__ T abs(const GpuComplex< T > &a_z) noexcept
@ t
fixed t as the independent variable
Definition ImpactXParticleContainer.H:38
amrex::SmallMatrix< amrex::ParticleReal, 6, 6, amrex::Order::F, 1 > Map6x6
Definition CovarianceMatrix.H:20
__host__ __device__ void make_invalid() const noexcept
static constexpr __host__ __device__ SmallMatrix< T, NRows, NCols, ORDER, StartIndex > Identity() noexcept
Definition ReferenceParticle.H:33
static constexpr Shape DEFAULT_shape
Definition Aperture.H:59
ImpactXParticleContainer::ParticleType PType
Definition Aperture.H:44
static constexpr amrex::ParticleReal DEFAULT_repeat_y
Definition Aperture.H:57
static constexpr auto type
Definition Aperture.H:43
AMREX_GPU_HOST_DEVICE AMREX_FORCE_INLINE void operator()(T_Real const &AMREX_RESTRICT x, T_Real const &AMREX_RESTRICT y, T_Real const &AMREX_RESTRICT t, T_Real const &AMREX_RESTRICT px, T_Real const &AMREX_RESTRICT py, T_Real const &AMREX_RESTRICT pt, T_IdCpu &AMREX_RESTRICT idcpu, RefPart const &AMREX_RESTRICT refpart) const
Definition Aperture.H:145
static constexpr Action DEFAULT_action
Definition Aperture.H:60
AMREX_ENUM_IN_CLASS(Shape, rectangular, elliptical,)
void compute_constants(RefPart const &refpart)
Definition Aperture.H:126
amrex::ParticleReal m_repeat_x
maximum vertical coordinate
Definition Aperture.H:280
static constexpr amrex::ParticleReal DEFAULT_repeat_x
Definition Aperture.H:56
amrex::ParticleReal m_inv_aperture_y
maximum horizontal coordinate
Definition Aperture.H:279
AMREX_GPU_HOST AMREX_FORCE_INLINE Map6x6 transport_map(RefPart const &AMREX_RESTRICT refpart) const
Definition Aperture.H:267
Shape m_shape
Definition Aperture.H:273
AMREX_ENUM_IN_CLASS(Action, transmit, absorb,)
amrex::ParticleReal m_inv_aperture_x
action type (transmit, absorb)
Definition Aperture.H:278
Aperture(amrex::ParticleReal aperture_x, amrex::ParticleReal aperture_y, amrex::ParticleReal repeat_x=DEFAULT_repeat_x, amrex::ParticleReal repeat_y=DEFAULT_repeat_y, bool shift_odd_x=DEFAULT_shift_odd_x, Shape shape=DEFAULT_shape, Action action=DEFAULT_action, amrex::ParticleReal dx=DEFAULT_dx, amrex::ParticleReal dy=DEFAULT_dy, amrex::ParticleReal rotation_degree=DEFAULT_rotation_degree, std::optional< std::string > name=DEFAULT_name)
Definition Aperture.H:77
bool m_shift_odd_x
vertical period for repeated masking
Definition Aperture.H:282
Action m_action
aperture type (rectangular, elliptical)
Definition Aperture.H:274
static constexpr bool DEFAULT_shift_odd_x
Definition Aperture.H:58
amrex::ParticleReal m_repeat_y
horizontal period for repeated masking
Definition Aperture.H:281
void reverse()
Definition Aperture.H:114
AMREX_GPU_HOST_DEVICE AMREX_FORCE_INLINE void spin_and_phasespace_push(T_Real const &AMREX_RESTRICT x, T_Real const &AMREX_RESTRICT y, T_Real const &AMREX_RESTRICT t, T_Real const &AMREX_RESTRICT px, T_Real const &AMREX_RESTRICT py, T_Real const &AMREX_RESTRICT pt, T_Real const &AMREX_RESTRICT sx, T_Real const &AMREX_RESTRICT sy, T_Real const &AMREX_RESTRICT sz, T_IdCpu &AMREX_RESTRICT idcpu, RefPart const &AMREX_RESTRICT refpart) const
Definition Aperture.H:233
Definition alignment.H:29
AMREX_GPU_HOST_DEVICE AMREX_FORCE_INLINE amrex::ParticleReal dy() const
Definition alignment.H:193
static constexpr amrex::ParticleReal DEFAULT_dy
Definition alignment.H:34
static constexpr amrex::ParticleReal DEFAULT_dx
Definition alignment.H:33
AMREX_GPU_HOST_DEVICE AMREX_FORCE_INLINE amrex::ParticleReal dx() const
Definition alignment.H:183
AMREX_GPU_HOST AMREX_FORCE_INLINE Map6x6 rotate_aligned_map(Map6x6 const &R) const
Definition alignment.H:267
Alignment(amrex::ParticleReal dx, amrex::ParticleReal dy, amrex::ParticleReal rotation_degree)
Definition alignment.H:43
static constexpr amrex::ParticleReal DEFAULT_rotation_degree
Definition alignment.H:35
Definition beamoptic.H:567
Definition lineartransport.H:50
static constexpr std::nullopt_t DEFAULT_name
Definition named.H:30
AMREX_GPU_HOST Named(std::optional< std::string > name)
Definition named.H:59
AMREX_FORCE_INLINE std::string name() const
Definition named.H:124
Definition nofinalize.H:22
Definition spintransport.H:36